/*
 * Starter sample for fallback fonts
 *
 * Required software: PDFlib/PDFlib+PDI/PPS 9
 * Required data: suitable fonts, Japanese CMaps
 */

import com.pdflib.pdflib;
import com.pdflib.PDFlibException;

class starter_fallback {
    public static void main(String argv[]) {
        /* This is where the data files are. Adjust as necessary. */
        final String searchpath = "../data";
        final String outfile = "starter_fallback.pdf";

        String optlist;
        pdflib p = null;
        int i, table;
        final double llx = 30, lly = 50, urx = 800, ury = 570;
        String result;
        int exitcode = 0;

        final String headers[] = { "Use case",
            "Option list for the 'fallbackfonts' option", "Base font",
            "With fallback font" };

        class testcase {
            testcase(String usecase, String fontname, String encoding,
                    String fallbackoptions, String text) {
                this.usecase = usecase;
                this.fontname = fontname;
                this.encoding = encoding;
                this.fallbackoptions = fallbackoptions;
                this.text = text;
            }

            String usecase;
            String fontname;
            String encoding;
            String fallbackoptions;
            String text;
        }

        final testcase testcases[] = {
            /* Add Euro glyph to an encoding which doesn't support it */
            new testcase(
                "Extend 8-bit encoding", "Helvetica", "iso8859-1",
                "{fontname=Helvetica encoding=unicode forcechars=euro}",
                /*
                 * Reference Euro glyph by name (since it is missing from
                 * the encoding)
                 */
                "123&euro;"),
            
            new testcase(
                "Use Euro glyph from another font",
                "Courier",
                "unicode",
                "{fontname=Helvetica encoding=unicode forcechars=euro textrise=-5%}",
                "123&euro;"),

            new testcase(
                "Add enlarged pictogram", "NotoSerif-Regular", "unicode",
                /* U+261E = WHITE RIGHT POINTING INDEX  */
                "{fontname=ZapfDingbats encoding=unicode forcechars=U+261E fontsize=150% "
                        + "textrise=-15%}", "hand symbol: \u261E"),

            new testcase(
                "Add enlarged symbol glyph",
                "NotoSerif-Regular",
                "unicode",
                "{fontname=Symbol encoding=unicode forcechars=U+2663 fontsize=125%}",
                "club symbol: \u2663"),
                    
            /*
             * Hebrew characters missing in the font will be pulled from Hebrew
             * font
             */
            new testcase(
                "Add Hebrew characters to Latin font", "NotoSerif-Regular",
                "unicode", "{fontname=NotoSerifHebrew-Regular encoding=unicode}",
                "Hebrew: \u05D0"),

            /* Font with end-user defined character (EUDC) */
            new testcase(
                "Gaiji with EUDC font", "KozMinProVI-Regular",
                "unicode",
                "{fontname=EUDC encoding=unicode forcechars=U+E000 fontsize=140% "
                        + "textrise=-20%}", "Gaiji: \uE000"),

            new testcase(
                "Replace Latin characters in CJK font",
                "KozMinProVI-Regular",
                "unicode",
                "{fontname=Courier encoding=unicode forcechars={U+0020-U+007E}}",
                "Latin and \u65E5\u672C\u8A9E")
        };

        try {
            p = new pdflib();

            p.set_option("searchpath={" + searchpath + "}");
            p.set_option("charref=true");
            p.set_option("glyphcheck=replace");

            /*
             * This means that formatting and other errors will raise an
             * exception. This simplifies our sample code, but is not
             * recommended for production code.
             */
            p.set_option("errorpolicy=exception");

            /* Set an output path according to the name of the topic */
            if (p.begin_document(outfile, "") == -1) {
                throw new Exception("Error: " + p.get_errmsg());
            }
            p.set_info("Creator", "PDFlib starter sample");
            p.set_info("Title", "starter_fallback");

            /* Start Page */
            p.begin_page_ext(0, 0, "width=a4.height height=a4.width");

            table = -1;

            /* Table header */
            for (i = 0; i < headers.length; i++) {
                final int col = i + 1;

                optlist = "fittextline={fontname=NotoSerif-Regular encoding=unicode fontsize=10} "
                        + "margin=4";
                table = p.add_table_cell(table, col, 1, headers[i], optlist);
            }

            /* Create fallback samples, one use case per row */
            for (i = 0; i < testcases.length; i++) {
                final int row = i + 2;
                final testcase testcase = testcases[i];
                int col = 1;

                /* Column 1: description of the use case */
                optlist = "fittextline={fontname=NotoSerif-Regular encoding=unicode fontsize=10} "
                        + "margin=4";
                table = p.add_table_cell(table, col++, row, testcase.usecase,
                        optlist);

                /* Column 2: reproduce option list literally */
                optlist = "fittextline={fontname=NotoSerif-Regular encoding=unicode fontsize=10} "
                        + "margin=4";
                table = p.add_table_cell(table, col++, row,
                        testcase.fallbackoptions, optlist);

                /* Column 3: text with base font */
                optlist = "fittextline={fontname=" + testcase.fontname
                        + " encoding=" + testcase.encoding
                        + " fontsize=10 replacementchar=? } margin=4";
                table = p.add_table_cell(table, col++, row, testcase.text,
                        optlist);

                /* Column 4: text with base font and fallback fonts */
                optlist = "fittextline={fontname=" + testcase.fontname
                        + " encoding=" + testcase.encoding
                        + " fontsize=10 fallbackfonts={"
                        + testcase.fallbackoptions + "}} margin=4";
                table = p.add_table_cell(table, col++, row, testcase.text,
                        optlist);
            }

            /* Place the table */
            optlist = "header=1 fill={{area=rowodd "
                    + "fillcolor={gray 0.9}}} stroke={{line=other}} ";
            result = p.fit_table(table, llx, lly, urx, ury, optlist);

            if (result.equals("_error")) {
                throw new Exception("Couldn't place table: " + p.get_errmsg());
            }

            p.end_page_ext("");
            p.end_document("");
        }
        catch (PDFlibException e) {
            System.err.println("PDFlib exception occurred:");
            System.err.println("[" + e.get_errnum() + "] " + e.get_apiname() +
                ": " + e.get_errmsg());
	    exitcode = 1;
        }
        catch (Exception e) {
            System.err.println(e);
	    exitcode = 1;
        }
        finally {
            if (p != null) {
                p.delete();
            }
	    System.exit(exitcode);
        }
    }
}